@@ -1,9 +1,9 @@
 package PAR::Packer;
-use 5.008001;
+use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '1.063_001';
+our $VERSION = '1.001';
 
 =head1 NAME
 
@@ -26,16 +26,15 @@ have a C compiler.
 
 use Config;
 use Archive::Zip ();
-use ExtUtils::MakeMaker (); # just for maybe_command()
-use Cwd qw( abs_path );
-use File::Basename;
+use Cwd ();
+use ExtUtils::MakeMaker ();
+use File::Basename ();
 use File::Find ();
-use File::Spec::Functions qw( :ALL );
-use File::Temp qw( tempfile );
+use File::Spec ();
+use File::Temp ();
 use Module::ScanDeps ();
 use PAR ();
 use PAR::Filter ();
-use PAR::Filter::PodStrip ();
 
 use constant OPTIONS => {
     'a|addfile:s@'   => 'Additional files to pack',
@@ -48,12 +47,13 @@ use constant OPTIONS => {
     'e|eval:s'       => 'Packing one-liner',
     'E|evalfeature:s'=> 'Packing one-liner with new syntactic features',
     'x|execute'      => 'Execute code to get dependencies',
-    'xargs:s'        => 'Args to pass when executing code',
     'X|exclude:s@'   => 'Exclude modules',
     'f|filter:s@'    => 'Input filters for scripts',
     'g|gui'          => 'No console window',
+    'i|icon:s'       => 'Icon file',
+    'N|info:s@'      => 'Executable header info',
     'I|lib:s@'       => 'Include directories (for perl)',
-    'l|link:s@'      => 'Additional shared libraries to pack',
+    'l|link:s@'      => 'Include additional shared libraries',
     'L|log:s'        => 'Where to log packaging process information',
     'F|modfilter:s@' => 'Input filter for perl modules',
     'M|module|add:s@'=> 'Include modules',
@@ -67,7 +67,6 @@ use constant OPTIONS => {
     'S|save'         => 'Preserve intermediate PAR files',
     's|sign'         => 'Sign PAR files',
     'T|tempcache:s'  => 'Temp cache name',
-    'u|unicode'      => 'Include Unicode stuff',
     'v|verbose:i'    => 'Verbosity level',
     'vv|verbose2',   => 'Verbosity level 2',
     'vvv|verbose3',  => 'Verbosity level 3',
@@ -108,7 +107,8 @@ sub set_options {
 #    $self->{parl} ||= $self->_extract_parl('PAR::StrippedPARL::Static')
 #      or die("Can't find par loader");
 #   $self->{parl_is_temporary} = 1;
-    $self->{dynperl} ||= $Config{useshrplib} && ($Config{useshrplib} ne 'false');
+    $self->{dynperl} ||=
+      $Config{useshrplib} && ($Config{useshrplib} ne 'false');
     $self->{script_name} = $opt{script_name} || $0;
 }
 
@@ -191,6 +191,9 @@ sub _check_write {
         if (-e _) {
             $self->_die("Cannot write on $file: $!\n") unless -w _;
         }
+        unless (-w Cwd::cwd()) {
+            $self->_die("Cannot write in this directory: $!\n");
+        }
     }
 }
 
@@ -296,18 +299,18 @@ sub _parse_opts {
     $self->{logfh} = $self->_open('>>', $opt->{L})
       if length $opt->{L};
 
-    if (defined $opt->{E}) {
+    if ($opt->{E}) {
         $opt->{e} = "use $];\n#line 1\n$opt->{E}";
         # XXX This is how we should also include additional default modules in the future instead of in require_modules in par.pl!
         push @{ $opt->{M} ||= [] }, 'feature' if $] >= 5.009;
     }
 
-    if (defined $opt->{e}) {
+    if ($opt->{e}) {
         $self->_warn("Using -e 'code' as input file, ignoring @$args\n")
           if (@$args and !$opt->{r});
 
-        my ($fh, $fake_input) = tempfile(
-	    "ppXXXXX", SUFFIX => ".pl", TMPDIR => 1, UNLINK => 1);
+        my ($fh, $fake_input) =
+          File::Temp::tempfile("ppXXXXX", SUFFIX => ".pl", UNLINK => 1);
 
         print $fh $opt->{e};
         close $fh;
@@ -358,7 +361,7 @@ sub _create_valid_hash {
     return () if (%$hashout);
 
     foreach my $key (keys(%$hashin)) {
-        my (@keys) = $key =~ /(?<!:)(\w+)/g;
+        my (@keys) = ($key =~ /\|/) ? ($key =~ /(?<!:)(\w+)/g) : ($key);
         @{$hashout}{@keys} = ($hashin->{$key}) x @keys;
     }
 }
@@ -436,8 +439,8 @@ sub run_pack {
     my $output = $self->{output};
     my $args   = $self->{args};
 
-    if (!file_name_is_absolute($output)) {
-	$output = catfile(".", $output);
+    if (!File::Spec->file_name_is_absolute($output)) {
+	$output = File::Spec->catfile(".", $output);
     }
 
     my @loader = ();
@@ -451,6 +454,7 @@ sub run_pack {
 sub _compile_par {
     my ($self) = @_;
 
+    my @SharedLibs;
     local (@INC) = @INC;
 
     my $lose = $self->{pack_attrib}{lose};
@@ -544,7 +548,6 @@ sub _make_manifest {
     my ($self) = @_;
 
     my $full_manifest = $self->{full_manifest};
-    my $add_manifest  = $self->{add_manifest};
 
     my $opt      = $self->{options};
     my $par_file = $self->{par_file};
@@ -556,7 +559,7 @@ sub _make_manifest {
 
     my $manifest = join("\n",
 '    <!-- accessible as jar:file:///NAME.par!/MANIFEST in compliant browsers -->',
-        (sort keys %$full_manifest, keys %$add_manifest),
+        (sort keys %$full_manifest),
 q(    # <html><body onload="var X=document.body.innerHTML.split(/\n/);var Y='<iframe src=&quot;META.yml&quot; style=&quot;float:right;height:40%;width:40%&quot;></iframe><ul>';for(var x in X){if(!X[x].match(/^\s*#/)&&X[x].length)Y+='<li><a href=&quot;'+X[x]+'&quot;>'+X[x]+'</a>'}document.body.innerHTML=Y">)
     );
 
@@ -567,7 +570,7 @@ conflicts: {}
 dist_name: $dist_name
 distribution_type: par
 dynamic_config: 0
-generated_by: '$class version $version'
+generated_by: '$class version $version
 license: unknown
 par:
   clean: $clean
@@ -611,7 +614,7 @@ sub get_par_file {
     if ($opt->{S} or $opt->{p}) {
         # We need to keep it.
 
-        if (defined $opt->{e} or !@$input) {
+        if ($opt->{e} or !@$input) {
             $par_file = (defined $output ? $output : "a.par");
         }
         else {
@@ -630,8 +633,7 @@ sub get_par_file {
         # Don't need to keep it, be safe with a tempfile.
 
         $self->{pack_attrib}{lose} = 1;
-        ($cfh, $par_file) = tempfile(
-	    "ppXXXXX", SUFFIX => ".par", TMPDIR => 1, UNLINK => 1);
+        ($cfh, $par_file) = File::Temp::tempfile("ppXXXXX", SUFFIX => ".par");
         close $cfh;    # See comment just below
     }
     $self->{par_file} = $par_file;
@@ -667,42 +669,16 @@ sub pack_manifest_hash {
     $root = "$Config{archname}/" if ($opt->{m});
     $self->{pack_attrib}{root} = '';
 
+    my @mscandeps_cache;
+    if (defined $opt->{cachedeps}) {
+        @mscandeps_cache = (cache_file => $opt->{cachedeps});
+    }
+
     my $par_file = $self->{par_file};
     my (@modules, @data, @exclude);
 
-    # Search for scannable code in all -I'd paths
-    # NOTE: We need this early, because Module::ScanDeps::_find_in_inc
-    # and others refer to @IncludeLibs.
-    push @Module::ScanDeps::IncludeLibs, @{$opt->{I}} if $opt->{I};
-
     foreach my $name (@{ $opt->{M} || [] }) {
-        if ($name =~ s/^([\w:]+)::(\*{0,2})$/$1/) {
-            my $ns = length $2;        # namespace depth indicator
-
-            if ($ns == 0) {
-                # "-MFoo::" shorthand for "-MFoo -MFoo::**"
-                $self->_name2moddata($name, \@modules, \@data);
-                $ns = 2;
-            }
-
-            (my $mod = $name) =~ s/::/\//g;
-            my @mods_in_ns = $ns == 1
-                ? Module::ScanDeps::_glob_in_inc_1($mod, 1)
-                : Module::ScanDeps::_glob_in_inc($mod, 1);
-            $self->_name2moddata($_, \@modules, \@data) foreach @mods_in_ns;
-        }
-        else {
-            $self->_name2moddata($name, \@modules, \@data);
-        }
-    }
-
-    if ($opt->{u}) {
-        if ($] lt '5.031006') {
-            push @modules, "utf8_heavy.pl";
-        }
-        else {
-            $self->_warn("'-u' is not needed for Perl >= 5.31.6");
-        }
+        $self->_name2moddata($name, \@modules, \@data);
     }
 
     # Skip either
@@ -723,70 +699,67 @@ sub pack_manifest_hash {
     my %map;
 
     unshift(@INC, @{ $opt->{I} || [] });
-    unshift(@SharedLibs, map $self->_find_shlib($_), @{ $opt->{l} || [] });
+    unshift(@SharedLibs, map $self->_find_shlib($_, $sn), @{ $opt->{l} || [] });
 
-    # Note: _find_in_inc() may return ()
-    my %skip = map { $_ => 1 } grep { defined } map { Module::ScanDeps::_find_in_inc($_) } @exclude;
+    my $inc_find = $self->_obj_function($fe, '_find_in_inc');
+
+    my %skip = map { $_, 1 } map &$inc_find($_), @exclude;
     if ($^O eq 'MSWin32') {
-        %skip = (%skip, map { s{\\}{/}g; lc($_) => 1 } @SharedLibs);
+        %skip = (%skip, map { s{\\}{/}g; lc($_), 1 } @SharedLibs);
     }
     else {
-        %skip = (%skip, map { $_ => 1 } @SharedLibs);
+        %skip = (%skip, map { $_, 1 } @SharedLibs);
     }
 
     my $add_deps = $self->_obj_function($fe, 'add_deps');
 
-    if ($opt->{x} && defined $opt->{xargs}) {
-        require Text::ParseWords;
-        $opt->{x} = [ Text::ParseWords::shellwords($opt->{xargs}) ];
-    }
-
-    if (!$opt->{n}) {
-        my @files = @$input;
-
-        # apply %Preload to the -M'd modules and add them to the list of files to scan
-        foreach my $module (@modules) {
-            my $file = Module::ScanDeps::_find_in_inc($module)
-              or $self->_die("Cannot find module $module (specified with -M)\n");
-            push @files, $file;
-
-            my @preload = Module::ScanDeps::_get_preload($module) or next;
-
-            $add_deps->(
-                used_by => $file,
-                rv      => \%map,
-                modules => \@preload,
-                skip    => \%skip,
-            );
-        }
-
-        # run static scan on the augmented file list
-        $self->_obj_function($fe, 'scan_deps')->(
+    my @files; # files to scan
+    # Apply %Preload to the -M'd modules and add them to the list of
+    # files to scan
+    foreach my $module (@modules) {
+        my $file = &$inc_find($module)
+          or $self->_die("Cannot find module $module (specified with -M)\n");
+        push @files, $file;
+        
+        my $preload = Module::ScanDeps::_get_preload($module) or next;
+        
+        $add_deps->(
+            used_by => $file,
             rv      => \%map,
-            files   => \@files,
+            modules => $preload,
             skip    => \%skip,
+#            warn_missing => $args->{warn_missing},
+        );
+        push @files, map {&$inc_find($_)} @$preload;
+    }
+    push @files, @$input;
+
+    # Search for scannable code in all -I'd paths
+    push @Module::ScanDeps::IncludeLibs, @{$opt->{I}} if $opt->{I};
+    
+    my $scan_dispatch =
+      $opt->{n}
+      ? $self->_obj_function($fe, 'scan_deps_runtime')
+      : $self->_obj_function($fe, 'scan_deps');
+
+
+    $scan_dispatch->(
+        rv      => \%map,
+        files   => \@files,
+        execute => $opt->{x},
+        compile => $opt->{c},
+        skip    => \%skip,
+        @mscandeps_cache,
+        ($opt->{n}) ? () : (
             recurse => 1,
-            ($opt->{cachedeps}) ?  (cache_file => $opt->{cachedeps}) : (),
-        );
-    }
+            first   => 1,
+        ),
+    );
 
-    if ($opt->{c} || $opt->{x}) {
-        # run dynamic scan in the original $input files
-        $self->_obj_function($fe, 'scan_deps_runtime')->(
-            rv      => \%map,
-            files   => $input,
-            execute => $opt->{x},
-            compile => $opt->{c},
-            skip    => \%skip,
-            ($opt->{cachedeps}) ?  (cache_file => $opt->{cachedeps}) : (),
-        );
-    }
+    %skip = map { $_, 1 } map &$inc_find($_), @exclude;
+    %skip = (%skip, map { $_, 1 } @SharedLibs);
 
-    # Note: _find_in_inc() may return ()
-    %skip = map { $_ => 1 } grep { defined } map { Module::ScanDeps::_find_in_inc($_) } @exclude;
-    %skip = (%skip, map { $_ => 1 } @SharedLibs);
-
-    $add_deps->(
+    &$add_deps(
         rv      => \%map,
         modules => \@modules,
         skip    => \%skip,
@@ -804,7 +777,7 @@ sub pack_manifest_hash {
     my $size = 0;
     my $old_member;
 
-    if ($opt->{m} and -e $par_file) {
+    if ($opt->{'m'} and -e $par_file) {
         my $tmpzip = Archive::Zip->new();
         $tmpzip->read($par_file);
 
@@ -823,9 +796,8 @@ sub pack_manifest_hash {
     # generate a selective set of filters from the options passed in via -F
     my $mod_filter = _generate_filter($opt, 'F');
 
-    my ($privlib, $archlib) = map { (my $lib = $_) =~ s{\\}{/}g; $lib }
-                                  @Config{qw(privlibexp archlibexp)};
-
+    (my $privlib = $Config{privlib}) =~ s{\\}{/}g;
+    (my $archlib = $Config{archlib}) =~ s{\\}{/}g;
     foreach my $pfile (sort grep length $map{$_}, keys %map) {
         next if !$opt->{B} and (
             ($map{$pfile} eq "$privlib/$pfile") or
@@ -842,6 +814,21 @@ sub pack_manifest_hash {
             $dep_manifest->{ $root . "lib/$pfile" } =
               [ string => $content_ref ];
         }
+        elsif (
+            File::Basename::basename($map{$pfile}) =~ /^Tk\.dll$/i and $opt->{i}
+            and eval { require Win32::Exe; 1 }
+            and eval { require Win32::Exe::IconFile; 1 }
+            and 0 # XXX - broken on larger icon files - XXX
+        ) {
+            my $tkdll = Win32::Exe->new($map{$pfile});
+            my $ico = Win32::Exe::IconFile->new($opt->{i});
+            $tkdll->set_icons(scalar $ico->icons);
+
+            $full_manifest->{ $root . "lib/$pfile" } =
+              [ string => $tkdll->dump ];
+            $dep_manifest->{ $root . "lib/$pfile" } =
+              [ string => $tkdll->dump ];
+        }
         else {
             $full_manifest->{ $root . "lib/$pfile" } =
               [ file => $map{$pfile} ];
@@ -856,7 +843,7 @@ sub pack_manifest_hash {
 
     my $in;
     foreach my $in (@$input) {
-        my $name = basename($in);
+        my $name = File::Basename::basename($in);
 
         if ($script_filter) {
             my $string = $script_filter->apply($in, $name);
@@ -868,46 +855,13 @@ sub pack_manifest_hash {
             $full_manifest->{"script/$name"} = [ file => $in ];
             $dep_manifest->{"script/$name"}  = [ file => $in ];
         }
-
-        delete $full_manifest->{$root . "lib/$name"};
-        delete $dep_manifest->{$root . "lib/$name"};
     }
 
     my $shlib = "shlib/$Config{archname}";
 
     foreach my $in (@SharedLibs) {
         next unless -e $in;
-
-        my $name;
-
-        # try to find the name the runtime loader will be looking for
-        if ($^O =~ /linux|solaris|freebsd|openbsd/i) {
-            # try "objdump" to extract SONAME
-            my $od = qx( objdump -p $in );
-            if ($? == 0 && $od =~ /^\s* SONAME \s+ (\S+)/mx) {
-                $name = $1;
-                $self->_vprint(1, "... objdump: library $in has SONAME $name");
-            }
-        }
-        elsif ($^O eq 'darwin') {
-            # try "otool -D $file", expect itwo lines of output like
-            #   $file:
-            #   path
-            # Note: some versions of otool report just a name in the
-            # second line, others report a pathname
-            chomp(my @ot = qx( otool -D $in ));
-            if ($? == 0) {
-                $name = basename($ot[1]);
-                $self->_vprint(1, "... otool: library $in has install name $name");
-            }
-            else {
-                # fallback to old "chasing symlinks" method
-                $name = basename($self->_chase_lib_darwin($in));
-            }
-        }
-
-        # fallback to old "chasing symlinks" method if nothing else worked
-        $name = basename($self->_chase_lib($in)) unless defined $name;
+        my $name = File::Basename::basename($in);
 
         $dep_manifest->{"$shlib/$name"}  = [ file => $in ];
         $full_manifest->{"$shlib/$name"} = [ file => $in ];
@@ -925,7 +879,7 @@ sub pack_manifest_hash {
     if (@$input and (@$input == 1 or !$opt->{p})) {
         my $string =
           (@$input == 1)
-          ? $self->_main_pl_single("script/" . basename($input->[0]))
+          ? $self->_main_pl_single("script/" . File::Basename::basename($input->[0]))
           : $self->_main_pl_multi();
 
         $full_manifest->{"script/main.pl"} = [ string => $string ];
@@ -947,7 +901,7 @@ sub _generate_filter {
 
     my $verbatim = ($ENV{PAR_VERBATIM} || 0);
 
-    # List of filters. If the regex is undefined or matches the
+    # List of filters. If the regex is undefined or matches the 
     # file name (e.g. Foo/Bar.pm), apply filter to this module.
     my @filters = (
         { regex => undef, filter => PAR::Filter->new('PatchContent') },
@@ -960,6 +914,7 @@ sub _generate_filter {
             filter => PAR::Filter->new($filter)
         };
     }
+    my $podstrip = PAR::Filter->new('PodStrip');
 
     my $filtersub = sub {
         my $ref = shift;
@@ -977,7 +932,7 @@ sub _generate_filter {
 
         # PodStrip by default, overridden by -F or $ENV{PAR_VERBATIM}
         if ($filtered == 1 and not $verbatim) {
-            $ref = PAR::Filter::PodStrip->apply($ref, '');
+            $ref = $podstrip->apply($ref, $name);
         }
         return $ref;
     };
@@ -1042,30 +997,29 @@ sub _add_manifest {
     $files = $opt->{a} if ($opt->{a});
     $lists = $opt->{A} if ($opt->{A});
 
-    {
-        local $/ = "\n";
-        foreach my $list (@$lists) {
-            my $fh = $self->_open('<', $list, 'text');
-            while (my $line = <$fh>) {
-                chomp($line);
-                push(@$files, $line);
-            }
+    local $/ = "\n";
+    foreach my $list (@$lists) {
+        my $fh = $self->_open('<', $list, 'text');
+        while (my $line = <$fh>) {
+            chomp($line);
+            push(@$files, $line);
         }
     }
 
-    foreach my $src_dst (grep length, @$files) {
-        $src_dst =~ s{\\}{/}g;
+    foreach my $file (grep length, @$files) {
+        $file =~ s{\\}{/}g;
 
-        my ($src, $dst) = split(";", $src_dst, 2);
-        $dst = $src unless defined $dst;
-
-        $dst =~ s{^[a-zA-Z]:}{} if $^O eq 'MSWin32';
-        $dst =~ s{^/}{};
-        die qq[--addfile/--addlist target path "$src_dst" must not contain components "." or ".."]
-            if grep { /^\.\.?$/ } split("/", $dst);
-        push(@$return, [ $src, $dst ]);
+        if ($file =~ /;/) {
+            push(@$return, [ split(/;/, $file) ]);
+        }
+        else {
+            my $alias = $file;
+            $alias =~ s{^[a-zA-Z]:}{} if $^O eq 'MSWin32';
+            $alias =~ s{^/}{};
+            push(@$return, [ $file, $alias ]);
+        }
     }
-    return $return;
+    return ($return);
 }
 
 sub add_manifest {
@@ -1097,16 +1051,17 @@ sub _add_pack_manifest {
     my $root        = $self->{pack_attrib}{root};
     my $shared_libs = $self->{pack_attrib}{shared_libs};
 
-    $zip->addDirectory('', substr($root, 0, -1))->unixFileAttributes(0755) if ($root and %$map);
-    $zip->addDirectory('', $root . 'lib')->unixFileAttributes(0755) if %$map;
+    $zip->addDirectory('', substr($root, 0, -1))->unixFileAttributes(0755)
+      if ($root and %$map and $] >= 5.008);
+    $zip->addDirectory('', $root . 'lib')->unixFileAttributes(0755) if (%$map and $] >= 5.008);
 
     my $shlib = "shlib/$Config{archname}";
-    $zip->addDirectory('', $shlib)->unixFileAttributes(0755) if @$shared_libs;
+    $zip->addDirectory('', $shlib)->unixFileAttributes(0755) if (@$shared_libs and $] >= 5.008);
 
     my @tmp_input = @$input;
     @tmp_input = grep !/\.pm\z/i, @tmp_input;
 
-    $zip->addDirectory('', 'script')->unixFileAttributes(0755) if @tmp_input;
+    $zip->addDirectory('', 'script')->unixFileAttributes(0755) if (@tmp_input and $] >= 5.008);
 
     my $in;
     foreach $in (sort keys(%$pack_manifest)) {
@@ -1155,9 +1110,9 @@ sub _add_file {
                 $manifest->{ $alias } = [ file => $file ];
 
                 $oldsize += -s $file;
-                my $member = $zip->addFile($file, $alias);
-                $member->desiredCompressionMethod($method);
-                $member->desiredCompressionLevel($level);
+                $zip->addFile($file, $alias);
+                $zip->memberNamed($alias)->desiredCompressionMethod($method);
+                $zip->memberNamed($alias)->desiredCompressionLevel($level);
             }
         }
         elsif (-e $fn and -r $fn) {
@@ -1168,9 +1123,9 @@ sub _add_file {
             $self->_vprint(1, "... adding $fn as $in\n");
 
             $oldsize += -s $fn;
-            my $member = $zip->addFile($fn => $in);
-            $member->desiredCompressionMethod($method);
-            $member->desiredCompressionLevel($level);
+            $zip->addFile($fn => $in);
+            $zip->memberNamed($in)->desiredCompressionMethod($method);
+            $zip->memberNamed($in)->desiredCompressionLevel($level);
         }
     }
     else {
@@ -1182,10 +1137,9 @@ sub _add_file {
         $oldsize += length($str);
 
         $self->_vprint(1, "... adding <string> as $in");
-        my $member = $zip->addString($str => $in);
-        $member->unixFileAttributes(0644);
-        $member->desiredCompressionMethod($method);
-        $member->desiredCompressionLevel($level);
+        $zip->addString($str => $in)->unixFileAttributes(0644);
+        $zip->memberNamed($in)->desiredCompressionMethod($method);
+        $zip->memberNamed($in)->desiredCompressionLevel($level);
     }
 
     $self->{pack_attrib}{old_size} = $oldsize;
@@ -1300,10 +1254,29 @@ sub _par_to_exe {
     if ($^O ne 'MSWin32' or $opt->{p} or $opt->{P}) {
         $self->_generate_output();
     }
-    else {
+    elsif (!$opt->{N} and !$opt->{i}) {
         $self->_generate_output();
         $self->_fix_console() if $opt->{g};
     }
+    elsif (eval { require Win32::Exe; 1 }) {
+        $self->_move_parl();
+        Win32::Exe->new($self->{parl})->update(
+            icon => $opt->{i},
+            info => $opt->{N},
+        );
+
+        $self->_append_parl();
+        $self->_generate_output();
+
+        $self->_fix_console();
+        unlink($self->{parl});
+        unlink($self->{orig_parl});
+        unlink("$self->{parl}.bak");
+        return;
+    }
+    else {
+        $self->_die("--icon and --info support needs Win32::Exe");
+    }
 }
 
 # extracts a parl (static) or parldyn (dynamic) from the appropriate data class
@@ -1322,10 +1295,11 @@ sub _extract_parl {
       if not $class->can('write_parl');
 
     $self->_vprint(0, "Generating a fresh 'parl'.");
-    my ($fh, $filename) = tempfile(
-        "parlXXXXXXX", SUFFIX => $Config{_exe}, TMPDIR => 1, UNLINK => 1);
-    close $fh;
-
+    my ($fh, $filename) = File::Temp::tempfile(
+        "parlXXXXXXX",
+        SUFFIX => $Config{_exe},
+    );
+    
     my $success = $class->write_parl($filename);
     if (not $success) {
         $self->_die("Failed to extract a parl from '$class' to file '$filename'");
@@ -1336,6 +1310,34 @@ sub _extract_parl {
 }
 
 
+sub _fix_console {
+    my ($self) = @_;
+
+    my $opt      = $self->{options};
+    my $output   = $self->{output};
+    my $dynperl  = $self->{dynperl};
+
+    return unless $opt->{g};
+
+    $self->_vprint(1, "Fixing $output to remove its console window");
+    $self->_strip_console($output);
+
+    if ($dynperl and !$opt->{d}) {
+        # we have a static.exe that needs taking care of.
+        my $buf;
+        my $fh = $self->_open($self->{orig_parl} || $self->{parl});
+        seek $fh, -8, 2;
+        read $fh, $buf, 8;
+        die unless $buf eq "\nPAR.pm\n";
+        seek $fh, -12, 2;
+        read $fh, $buf, 4;
+        seek $fh, -12 - unpack("N", $buf) - 4, 2;
+        read $fh, $buf, 4;
+        close $fh;
+        $self->_strip_console($output, unpack("N", $buf));
+    }
+}
+
 sub _move_parl {
     my ($self) = @_;
 
@@ -1343,8 +1345,11 @@ sub _move_parl {
 
     my $cfh;
     my $fh = $self->_open($self->{parl});
-    ($cfh, $self->{parl}) = tempfile(
-        "parlXXXX", SUFFIX => ".exe", TMPDIR => 1, UNLINK => 1);
+    ($cfh, $self->{parl}) = File::Temp::tempfile(
+        "parlXXXX",
+        SUFFIX => ".exe",
+        UNLINK => 1,
+    );
     binmode($cfh);
 
     local $/;
@@ -1354,6 +1359,25 @@ sub _move_parl {
     $self->{fh} = $fh;
 }
 
+sub _append_parl {
+    my ($self) = @_;
+
+    my $fh = delete $self->{fh};
+
+    my $buf;
+    seek $fh, -8, 2;
+    read $fh, $buf, 8;
+    die unless $buf eq "\nPAR.pm\n";
+    seek $fh, -12, 2;
+    read $fh, $buf, 4;
+    seek $fh, -12 - unpack("N", $buf), 2;
+
+    local $/;
+    my $cfh = $self->_open('>>', $self->{parl});
+    print $cfh readline($fh);
+    close $cfh;
+}
+
 sub _generate_output {
     my ($self) = @_;
 
@@ -1365,7 +1389,7 @@ sub _generate_output {
     unshift @args, '-q' unless $opt->{v} > 0;
     if ($opt->{B}) {
         unshift @args, "-B";
-    }
+    } 
     if ($opt->{L}) {
         unshift @args, "-L".$opt->{L};
     }
@@ -1380,40 +1404,34 @@ sub _generate_output {
 
     # Make sure the parl is callable. Prepend ./ if it's just a file name.
     my $parl = $self->{parl};
-    my ($volume, $path, $file) = splitpath($parl);
+    my ($volume, $path, $file) = File::Spec->splitpath($parl);
     if (not defined $path or $path eq '') {
-        $parl = catfile(curdir(), $parl);
+        $parl = File::Spec->catfile(File::Spec->curdir(), $parl);
     }
 
     system($parl, @args);
 }
 
-sub _fix_console {
-    my ($self) = @_;
-
-    my $opt      = $self->{options};
-    my $output   = $self->{output};
-    my $dynperl  = $self->{dynperl};
-
-    return unless $opt->{g};
-
-    $self->_vprint(1, "Fixing $output to remove its console window");
+sub _strip_console {
+    my $self   = shift;
+    my $file   = shift;
+    my $preoff = shift || 0;
 
     my ($record, $magic, $signature, $offset, $size);
 
-    my $exe = $self->_open('+<', $output);
+    my $exe = $self->_open('+<', $file);
     binmode $exe;
-    seek $exe, 0, 0;
+    seek $exe, $preoff, 0;
 
     # read IMAGE_DOS_HEADER structure
     read $exe, $record, 64;
     ($magic, $offset) = unpack "Sx58L", $record;
 
-    die "$output is not an MSDOS executable file.\n"
+    die "$file is not an MSDOS executable file.\n"
       unless $magic == 0x5a4d;    # "MZ"
 
     # read signature, IMAGE_FILE_HEADER and first WORD of IMAGE_OPTIONAL_HEADER
-    seek $exe, $offset, 0;
+    seek $exe, $preoff + $offset, 0;
     read $exe, $record, 4 + 20 + 2;
 
     ($signature, $size, $magic) = unpack "Lx16Sx2S", $record;
@@ -1426,7 +1444,7 @@ sub _fix_console {
       ($size == 240 && $magic == 0x20b);    # IMAGE_NT_OPTIONAL_HDR64_MAGIC
 
     # Offset 68 in the IMAGE_OPTIONAL_HEADER(32|64) is the 16 bit subsystem code
-    seek $exe, $offset + 4 + 20 + 68, 0;
+    seek $exe, $preoff + $offset + 4 + 20 + 68, 0;
     print $exe pack "S", 2;                 # IMAGE_WINDOWS
     close $exe;
 }
@@ -1488,20 +1506,18 @@ sub _check_par {
 sub _chase_lib {
    my ($self, $file) = @_;
 
-   $file = abs_path($file);
-
-   while ($Config{d_symlink} and -l $file) {
+   while ($Config::Config{d_symlink} and -l $file) {
        if ($file =~ /^(.*?\.\Q$Config{dlext}\E\.\d+)\..*/) {
            return $1 if -e $1;
        }
 
        return $file if $file =~ /\.\Q$Config{dlext}\E\.\d+$/;
 
-       my $dir = dirname($file);
+       my $dir = File::Basename::dirname($file);
        $file = readlink($file);
 
-       unless (file_name_is_absolute($file)) {
-           $file = rel2abs($file, $dir);
+       unless (File::Spec->file_name_is_absolute($file)) {
+           $file = File::Spec->rel2abs($file, $dir);
        }
    }
 
@@ -1512,104 +1528,56 @@ sub _chase_lib {
    return $file;
 }
 
-sub _chase_lib_darwin {
-   my ($self, $file) = @_;
-
-   $file = abs_path($file);
-
-   while (-l $file) {
-       if ($file =~ /^(.*?\.\d+)(\.\d+)*\.dylib$/) {
-           my $name = $1 . q/.dylib/;
-           return $name if -e $name;
-       }
-
-       return $file if $file =~ /\D\.\d+\.dylib$/;
-
-       my $dir = dirname($file);
-       $file = readlink($file);
-
-       unless (file_name_is_absolute($file)) {
-           $file = rel2abs($file, $dir);
-       }
-   }
-
-   if ($file =~ /^(.*?\.\d+)(\.\d+)*\.dylib$/) {
-       my $name = $1 . q/.dylib/;
-       return $name if -e $name;
-   }
-
-   return $file;
-}
-
-
 sub _find_shlib {
-    my ($self, $file) = @_;
+    my ($self, $file, $script_name) = @_;
 
-    if (-e $file) {
-        my $abs_file = abs_path($file);
-        $self->_vprint(1, "... found library $file: $abs_file");
-        return $abs_file;
-    }
-
-    $self->_die("Shared library (option --link) doesn't exist: $file")
-        if $file =~ /[\/\\]/;
-
-    my @libpath;
     if ($^O eq 'MSWin32') {
-        @libpath = (path(), '.');       # cwd() is always implicitly searched
-    }
-    else {
-        # NOTE: libpth is actually supposed to be the path searched
-        # by the linker (ld) and *not* the path searched by the runtime
-        # loader (ld.so). But it's the best guess we've got.
-        @libpath = split(' ', $Config{libpth});
-
-        # add $ENV{LD_LIBRARY_PATH} (or equivalent) if defined
-        my $ldlibpath = $ENV{ $Config{ldlibpthname} };
-        unshift @libpath, split(/\Q$Config{path_sep}\E/, $ldlibpath)
-            if defined $ldlibpath;
-    }
-
-    if (my $lib = $self->_find_shlib_in_path($file, @libpath)) {
-        $self->_vprint(1, "... found library $file: $lib");
-        return $lib;
-    }
-
-    $self->_die("Can't find shared library (option --link): $file")
-        if $^O eq 'MSWin32' || $file =~ /^lib/;
-
-    # be extra magical and prepend "lib" to the filename
-    if (my $lib = $self->_find_shlib_in_path("lib$file", @libpath)) {
-        $self->_vprint(1, "... found library $file: $lib");
-        return $lib;
-    }
-
-    $self->_die("Can't find shared library (option --link): $file (also tried lib$file)");
-}
-
-sub _find_shlib_in_path
-{
-    my ($self, $file, @path) = @_;
-
-    my $dlext = $^O eq 'darwin' ? 'dylib' : $Config{dlext};
-    for my $dir (@path)
-    {
-        $dir = '.' if $dir eq '';
-        foreach my $p (catfile($dir, $file), catfile($dir, "$file.$dlext"))
-        {
-            return abs_path($p) if -e $p;
+        if ($file !~ /^[a-z]:/i) {
+            my $cwd = Cwd::cwd().'/';
+            $cwd =~ s{/.*}{} if $file =~ m{^[\\/]};
+            $file = $cwd.$file;
         }
     }
-    return;
+
+    return $self->_chase_lib($file) if -e $file;
+
+    my $libpthname;
+    if ($^O eq 'MSWin32') {
+        $libpthname = exists $ENV{PATH} ? $ENV{PATH} : undef;
+    }
+    else {
+        $libpthname = exists $ENV{ $Config{ldlibpthname} }
+                           ? $ENV{ $Config{ldlibpthname} } : undef;
+    }
+    if (not defined $libpthname) {
+        print "Can't find $file. Environment variable "
+          . ($^O eq 'MSWin32' ? 'PATH' : $Config{ldlibpthname})
+          . " does not exist.\n";
+        return;
+    }
+
+    $file = File::Basename::basename($file);
+    for my $dir (File::Basename::dirname($0),
+        split(/\Q$Config{path_sep}\E/, $libpthname))
+    {
+        my $abs = File::Spec->catfile($dir, $file);
+        return $self->_chase_lib($abs) if -e $abs;
+        $abs = File::Spec->catfile($dir, "$file.$Config{dlext}");
+        return $self->_chase_lib($abs) if -e $abs;
+    }
+
+    # be extra magical and prepend "lib" to the filename
+    return if $^O eq 'MSWin32';
+    return $self->_find_shlib("lib$file", $script_name) unless $file =~ /^lib/;
 }
 
 sub _can_run {
     my ($self, $command, $no_exec) = @_;
 
-    for my $dir (dirname($0),
+    for my $dir (File::Basename::dirname($0),
         split(/\Q$Config{path_sep}\E/, $ENV{PATH}))
     {
-        my $abs = catfile($dir, $command);
+        my $abs = File::Spec->catfile($dir, $command);
         return $abs if $no_exec or $abs = MM->maybe_command($abs);
     }
     return;
@@ -1617,7 +1585,7 @@ sub _can_run {
 
 sub _main_pl_multi {
     my ($self) = @_;
-
+   
     # insert code for @INC cleaning (in case of bundling core modules)
     my $clean_inc = $self->_main_pl_clean();
     # insert code for reusable apps
@@ -1652,7 +1620,7 @@ sub _open {
 
 sub _main_pl_single {
     my ($self, $file) = @_;
-
+    
     # insert code for @INC cleaning (in case of bundling core modules)
     my $clean_inc = $self->_main_pl_clean();
     # insert code for reusable apps
@@ -1709,26 +1677,21 @@ __REUSE_APP__
 sub _main_pl_clean {
     my $self = shift;
     my $opt = $self->{options};
-
+    
     my $clean_inc = '';
     if ($opt->{B}) { # bundle core modules
         # weed out all @INC entries
-        # use a canonicalized $ENV{PAR_TEMP}: this path was created by C code
-        # and may not be in canonical form (so that the match below will
-        # fail); case inpoint: some versions of FreeBSD have
-        #  #define P_tmpdir "/var/tmp/"
-        # in /usr/include/stdio.h (note the trailing slash)
         $clean_inc = <<'__CLEAN_INC__';
 # Remove everything but PAR hooks from @INC
 my %keep = (
     \&PAR::find_par => 1,
     \&PAR::find_par_last => 1,
 );
-my $par_temp_dir = File::Spec->catdir( $ENV{PAR_TEMP} );
+my $par_temp_dir = quotemeta( $ENV{PAR_TEMP} );
 @INC =
     grep {
         exists($keep{$_})
-        or $_ =~ /^\Q$par_temp_dir\E/;
+        or $_ =~ /^$par_temp_dir/;
     }
     @INC;
 __CLEAN_INC__
@@ -1769,23 +1732,19 @@ Audrey Tang E<lt>cpan@audreyt.orgE<gt>
 
 Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
-Roderich Schupp E<lt>roderich.schupp@gmail.comE<gt>
-
-You can write
+L<http://par.perl.org/> is the official PAR website.  You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
-Archives of the mailing list are available at
-E<lt>https://www.mail-archive.com/par@perl.org/E<gt> or E<lt>https://groups.google.com/g/perl.parE<gt>.
 
-Please submit bug reports to E<lt>https://github.com/rschupp/PAR-Packer/issuesE<gt>.
+Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright 2004-2010 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
+Copyright 2004-2009 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
-See F<LICENSE>.
+See L<http://www.perl.com/perl/misc/Artistic.html>
 
 =cut
